use crate::prelude::*;

use std::os::raw::c_void;
use std::ptr;

#[repr(C)]
#[derive(Debug, PartialEq, Eq)]
pub struct VkWriteDescriptorSetAccelerationStructureKHR {
    pub sType: VkStructureType,
    pub pNext: *const c_void,
    pub accelerationStructureCount: u32,
    pub pAccelerationStructures: *const VkAccelerationStructureKHR,
}

impl VkWriteDescriptorSetAccelerationStructureKHR {
    pub fn new(acceleration_structures: &[VkAccelerationStructureKHR]) -> Self {
        VkWriteDescriptorSetAccelerationStructureKHR {
            sType: VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_ACCELERATION_STRUCTURE_KHR,
            pNext: ptr::null(),
            accelerationStructureCount: acceleration_structures.len() as u32,
            pAccelerationStructures: if acceleration_structures.is_empty() {
                ptr::null()
            } else {
                acceleration_structures.as_ptr()
            },
        }
    }

    pub fn set_acceleration_structures<'a>(
        &'a mut self,
        acceleration_structures: &'a [VkAccelerationStructureKHR],
    ) {
        debug_assert!(!acceleration_structures.is_empty());

        self.accelerationStructureCount = acceleration_structures.len() as u32;
        self.pAccelerationStructures = acceleration_structures.as_ptr();
    }
}

impl Default for VkWriteDescriptorSetAccelerationStructureKHR {
    fn default() -> Self {
        Self::new(&[])
    }
}
